/*
This file is part of the Juju GUI, which lets users view and manage Juju
environments within a graphical interface (https://launchpad.net/juju-gui).
Copyright (C) 2017 Canonical Ltd.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License version 3, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
General Public License for more details.

You should have received a copy of the GNU Affero General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

'use strict';

YUI.add('account-payment-method-card', function () {

  juju.components.AccountPaymentMethodCard = React.createClass({
    displayName: 'AccountPaymentMethodCard',

    propTypes: {
      addNotification: React.PropTypes.func,
      card: React.PropTypes.object.isRequired,
      onPaymentMethodRemoved: React.PropTypes.func,
      removePaymentMethod: React.PropTypes.func,
      updatePaymentMethod: React.PropTypes.func,
      username: React.PropTypes.string
    },

    getInitialState: function () {
      this.xhrs = [];
      return {
        cardFlipped: false
      };
    },

    componentWillUnmount: function () {
      this.xhrs.forEach(xhr => {
        xhr && xhr.abort && xhr.abort();
      });
    },

    /**
      Generate the card logo.
       @method _generateLogo
    */
    _generateLogo: function () {
      // Convert the brand to the format required for the logo.
      const brand = this.props.card.brand.toLowerCase().replace(' ', '-');
      return React.createElement(juju.components.SvgIcon, {
        size: '40',
        name: `card-${brand}` });
    },

    /**
      Handle toggling the card state.
       @method _handleCardClick
      @param {Object} e The click event from the checkbox.
    */
    _handleCardClick: function (e) {
      e.stopPropagation();
      this.setState({ cardFlipped: !this.state.cardFlipped });
    },

    /**
      Remove the payment method.
       @method _removePaymentMethod
    */
    _removePaymentMethod: function () {
      const xhr = this.props.removePaymentMethod(this.props.username, this.props.card.id, error => {
        if (error) {
          const message = 'Unable to remove the payment method';
          this.props.addNotification({
            title: message,
            message: `${message}: ${error}`,
            level: 'error'
          });
          console.error(message, error);
          return;
        }
        this.props.onPaymentMethodRemoved();
      });
      this.xhrs.push(xhr);
    },

    /**
      Generate the card actions.
       @method _generateActions
    */
    _generateActions: function () {
      if (!this.props.removePaymentMethod) {
        return null;
      }
      return React.createElement(
        'div',
        { className: 'four-col last-col account__payment-card-actions' },
        React.createElement(juju.components.GenericButton, {
          action: this._removePaymentMethod,
          type: 'inline-base',
          title: 'Remove payment details' }),
        React.createElement(juju.components.GenericButton, {
          action: this.props.updatePaymentMethod,
          type: 'inline-base',
          title: 'Update payment details' })
      );
    },

    render: function () {
      const card = this.props.card;
      const cardClasses = classNames('account__payment-card-wrapper', { 'account__payment-card-wrapper--flipped': this.state.cardFlipped });
      const address = card.address;
      const line2 = address.line2 ? React.createElement(
        'p',
        null,
        address.line2
      ) : null;
      return React.createElement(
        'div',
        { className: 'account__payment-card twelve-col' },
        React.createElement(
          'div',
          { className: 'eight-col' },
          React.createElement(
            'div',
            { className: cardClasses,
              onClick: this._handleCardClick },
            React.createElement(
              'div',
              { className: 'account__payment-card-container' },
              React.createElement(
                'div',
                { className: 'account__payment-card-front' },
                React.createElement('div', { className: 'account__payment-card-overlay' }),
                React.createElement(
                  'div',
                  { className: 'account__payment-card-name' },
                  card.cardHolder
                )
              ),
              React.createElement(
                'div',
                { className: 'account__payment-card-back' },
                React.createElement('div', { className: 'account__payment-card-overlay' }),
                React.createElement(
                  'div',
                  { className: 'account__payment-card-number' },
                  'xxxx xxxx xxxx ',
                  card.last4
                ),
                React.createElement(
                  'div',
                  { className: 'account__payment-card-bottom' },
                  React.createElement(
                    'div',
                    { className: 'account__payment-card-expiry' },
                    card.month,
                    '/',
                    card.year
                  ),
                  React.createElement(
                    'div',
                    { className: 'account__payment-card-brand' },
                    this._generateLogo()
                  )
                )
              )
            )
          ),
          React.createElement(
            'div',
            { className: 'account__payment-card-info' },
            React.createElement(
              'h4',
              null,
              'Card address'
            ),
            React.createElement(
              'p',
              null,
              address.line1
            ),
            line2,
            React.createElement(
              'p',
              null,
              address.city,
              ' ',
              address.state
            ),
            React.createElement(
              'p',
              null,
              address.country,
              ' ',
              address.postcode
            )
          )
        ),
        this._generateActions()
      );
    }

  });
}, '', {
  requires: ['generic-button', 'svg-icon']
});